#include <stdlib.h>
#include <iostream>

#include "configParser.hh"
#include "reductionAlgs.hh"

#define EXIT_USAGE 2
#define EXIT_OTHER_ERROR 3


// execv() needs char**'s, but those are hard to work with, so Lithium uses vector<string> internally and converts to char** when needed using this function.
char **svToCharStarStar (vector<string> &sv)
{
  unsigned int n = sv.size();
  char **css = new char* [n + 1];
  for (unsigned int i = 0; i < n; ++i)
    css[i] = strdup(sv[i].c_str());
  css[n] = (char*)0;
  return css;
}


int main(unsigned int argc, char **argv)
{
  if (argc < 5) {
    cerr << "Not enough arguments." << endl;
    cerr << endl;
    cerr << "Usage: ./lithium parseMode testcaseFilename target testName [testParams]" << endl;
    cerr << "e.g.   ./lithium L foo.html firefox crashes" << endl;
    cerr << endl;
    cerr << "parseMode can be C (chars), L (lines), or TIY (homogenous tree)." << endl;
    cerr << "testName can be 'crashes', 'hangs', 'logs', or 'leaks'; see the file 'tests'" << endl;
    cerr << "target can be 'firefox' or 'firefoxdebug'; see the file 'targets'" << endl;
    cerr << endl;
    cerr << "See the readme for more detailed hints." << endl;
    cerr << endl;
    return EXIT_USAGE;
  }
  
  char *testcaseMode = argv[1];
  char *testcaseFilename = argv[2];
  char *targetShortcut = argv[3];
  char *testShortcut = argv[4];
  
  vector<string> *targetV = findConfigLine("targets", targetShortcut);
  if (!targetV) {
    cerr << "Can't find information about target '" << targetShortcut << "' in file 'targets'" << endl;
    return EXIT_OTHER_ERROR;
  }
  if (targetV->size() != 1) {
    cerr << "There should only be one word following the target name in file 'targets'" << endl;
    delete targetV;
    return EXIT_OTHER_ERROR;
  }
  string target = (*targetV)[0];
  delete targetV;

  vector<string> *testV = findConfigLine("tests", testShortcut);
  if (!testV) {
    cerr << "Can't find information about test '" << testShortcut << "' in file 'tests'" << endl;
    return EXIT_OTHER_ERROR;
  }
  int desiredExitCode = atoi((*testV)[0].c_str());
  string testFilename = (*testV)[1];

  vector<string> paramsToTest;
  paramsToTest.push_back(""); // dunno why.

  unsigned int argvIndex = 5;
  for (unsigned int i = 2; i < testV->size(); ++i) {
    string s = (*testV)[i];
    if (s.length() > 0 && s[0] == '?') {
      if (argvIndex == argc) {
        cerr << "Not enough params for test '" << testShortcut << "' given on Lithium's command line" << endl;
        // would be nice to list the params needed
        return EXIT_USAGE;
      }
      cout << "Test param '" << s.substr(1, string::npos) << "': " << argv[argvIndex] << endl;
      paramsToTest.push_back(argv[argvIndex]);      
      ++argvIndex;
    } else {
      paramsToTest.push_back(s);
    }
  }
  
  if (argvIndex != argc) {
    cerr << "Too many extra arguments on command line for the given test" << endl;
    return EXIT_USAGE;
  }

  paramsToTest.push_back(target);
  paramsToTest.push_back(testcaseFilename);

  
  // leaky leaky but i don't care
  programInfo p(strdup(testFilename.c_str()), svToCharStarStar(paramsToTest));

  testcase t;
  t.setParseModes(testcaseMode);
  t.readFromFile(testcaseFilename);
  //t.show();
  t.writeToFile("testcase-backup");
  
  if (t.parts.size() < 2) {
    cerr << "Testcase has fewer than 2 parts." << endl;
    cerr << "Perhaps you need to run mac2unix on it, or make sure there are DDBEGIN and DDEND lines?" << endl;
    return EXIT_OTHER_ERROR;
  }


  cout << "Just checking to make sure the original testcase shows the problem." << endl;
  int actualExitCode = testTargetWithTestcase(p, t);
  cout << "Desired exit code: " << desiredExitCode << endl;
  cout << "Actual exit code: " << actualExitCode << endl;

  if (desiredExitCode == actualExitCode) {
    reduceTestcase(p, t, desiredExitCode);
  } else {
    cout << "Exit code was not as expected.  Do what you need to fix this, and run Lithium again." << endl;
    return EXIT_OTHER_ERROR;
  }
  
  return 0;
}
